/*-----------------------------------------------------------------------

File  : can_infstate.h

Author: Stephan Schulz

Contents
 
  Data structures for representing the state of an inference process. 

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Fri Feb 26 23:25:00 MET 1999
    New

-----------------------------------------------------------------------*/

#ifndef CAN_INFSTATE

#define CAN_INFSTATE

#include <ccl_inferencedoc.h>
#include <can_clausestore.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/


/* Represent the state of an inference process. This is a kind of
   strange date structure, because it represents the _result_ of a
   series of inference steps, not the steps themselves. */

typedef struct infstatecell
{
   TB_p      terms;
   PDArray_p clause_store;
   PDArray_p term_clause_assoc;
   NumTree_p watch_clauses;
   bool      print_clause_evals; /* Print clauses on parsing a CE
				    statement */
   bool      print_clause_mods;  /* Print clauses whenever they are
				    changed or created by an inference
				    */
   bool      print_clause_stats; /* Print clause statistics with
				    clause - only for clause output
				    during parsing */
}InfStateCell, *InfState_p;


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

#define InfStateCellAlloc()\
        (InfStateCell*)SizeMalloc(sizeof(InfStateCell))
#define InfStateCellFree(junk)\
        SizeFree(junk, sizeof(InfStateCell))

InfState_p InfStateAlloc();
void       InfStateFree(InfState_p state);

void       InfStateClausesDelProp(InfState_p state, ClauseProperties
				  props);
void       InfStateClausesSetProp(InfState_p state, ClauseProperties
				  props);

void       InfStateAddTermAssoc(InfState_p state, CompClause_p
				clause);
void       InfStateRemoveTermAssoc(InfState_p state, CompClause_p
				   clause);

void       InfStateCCParse(Scanner_p in, InfState_p state);
void       InfStateCPParse(Scanner_p in, InfState_p state);
void       InfStateCSParse(Scanner_p in, InfState_p state);
void       InfStateCMParse(Scanner_p in, InfState_p state);
void       InfStateRWParse(Scanner_p in, InfState_p state);

void       InfStateInferenceParse(Scanner_p in, InfState_p state);
long       InfStateInfListParse(Scanner_p in, InfState_p state);

void InfStatePrintClauses(FILE* out, InfState_p state,
			  ClauseProperties filter, bool status, bool
			  deps, bool stats);

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





