/*-----------------------------------------------------------------------

File  : ccl_rewrite.h

Author: Stephan Schulz

Contents
 
  Functions for rewriting terms and clauses with clause sets.

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Tue May 26 19:47:52 MET DST 1998
    New

-----------------------------------------------------------------------*/

#ifndef CCL_REWRITE

#define CCL_REWRITE

#include <cte_replace.h>
#include <ccl_pdtrees.h>
#include <ccl_clausesets.h>


/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/



extern long RewriteAttempts;
extern long RewriteSucesses;

#ifndef NDEBUG
extern bool RewriteWarnLeftSide;
#endif

Clause_p TermFindRewriteClause(OCB_p ocb, Term_p term, ClauseSet_p
			       demodulators, ClausePos_p pos, Subst_p
			       subst);

ClausePos_p TermIndexedFindDemodulator(OCB_p ocb, Term_p term, SysDate
				       date, ClauseSet_p demodulators,
				       Subst_p subst);

bool TermRewriteWithClauseSet(OCB_p ocb, TB_p bank, Term_p term,
			      SysDate date, ClauseSet_p demodulators);

bool TermRewriteWithClauseSetList(OCB_p ocb, TB_p bank, Term_p term,
				  ClauseSet_p* demodulators,
				  RewriteLevel level); 

bool TermComputeLINormalform(OCB_p ocb, TB_p bank, Term_p term,
			     ClauseSet_p *demodulators,
			     RewriteLevel level);

bool EqnComputeLINormalform(OCB_p ocb, TB_p bank, Eqn_p eqn,
			    ClauseSet_p *demodulators,
				  RewriteLevel level);

bool ClauseComputeLINormalform(OCB_p ocb, TB_p bank, Clause_p
			       clause, ClauseSet_p *demodulators,
				  RewriteLevel level);

bool ClauseSetComputeLINormalform(OCB_p ocb, TB_p bank, ClauseSet_p
				  set, ClauseSet_p *demodulators,
				  RewriteLevel level, bool res);

void FindClausesWithRewritableMaxSides(OCB_p ocb, ClauseSet_p set,
				       PTree_p* results, Clause_p
				       new_demod, SysDate nf_date);


#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





